#ifndef UNTITLED_CONSUMPTION_COUNTER_H
#define UNTITLED_CONSUMPTION_COUNTER_H

#include "Arduino.h"
#include "DateTime.h"
#include "PulseCounter.h"

//================Consumption class======================

/// \author silhavyj A17B0362P
///
/// This class is used to monitor daily, weekly, or monthly.
class Consumption {

//====================public data=======================
public:
    /// Type of the monitoring period. It is set through
    /// the constructure when creating an instance.
    enum Type {
        DAY,    ///< daily water consumption
        WEEK,   ///< weekly water consumption
        MONTH   ///< monthly water consumption
    };

//====================private data======================
private:
    PulseCounter *pulseCounter;  ///< instance of PulseCounter (input [l])
    RTCDateTime initialDateTime; ///< instance of RTCDateTime (real-time module)
    int count;                   ///< number of pulses detected within the monitoring period
    Type type;                   ///< type of the monitoring period

//==================public methods======================
public:
    /// Constructor of the class
    /// \param pulseCounter instance of PulseCounter (input)
    /// \param type of the monitoring period
    Consumption(PulseCounter *pulseCounter, Type type);

    /// Updates the variables
    ///
    /// If a pulse has just been detected, it will increment the counter.
    /// Also, if the monitoring period is over, it will reset the counter.
    /// This method is supposed to be called as frequently as possible
    /// in order to achieve precise results.
    void update();

    /// Returns the current number of pulses detected so far within the monitoring period.
    /// \return number of pulses detected so far
    int getConsumptionCount() const;
};

#endif