#ifndef HTML_DATA_SOURCE_H
#define HTML_DATA_SOURCE_H

#include "Arduino.h"

//==============HTMLDataSource class====================

/// \author silhavyj A17B0362P
///
/// This class works as an abstract class/interface
/// for all classes that are registered as a source
/// of data for the HTML content. All these classes
/// must implement the method(s) defined within this
/// interface.
class HTMLDataSource {
public:
    const static String UNDEFINED_DATA; ///< string "UNDEFINED"

public:
    /// If a class is registered as a source of data for the
    /// HTML content, it needs return the appropriate value 
    /// according to the id given as a parameter.
    /// 
    /// \param id - the id of the piece of data
    /// \return If the class holds this piece of information, it will
    ///         return it. Otherwise, "UNDEFINED" will be returned.
    virtual const String getHTMLData(const int id) const = 0;
};

#endif