#ifndef LEAK_DETECTION_CONFIG_H
#define LEAK_DETECTION_CONFIG_H

#include "DateTime.h"
#include "PulseCounter.h"

//========LeakDetectionConfig_t structure===============

/// \author silhavyj A17B0362P
///
/// This structure represents settings on a water leak
/// detection algorithm. This template is used as both
/// the normal and the alarm config. Not all the algorithms
/// have to use all the parameters. For instance, the
/// high-water leak detection algorithm doesn't use 
/// 'limitActionTime'.
struct LeakDetectionConfig_t {  
    time_t limitResetTime;  ///< limit reset time e.g. 5 mins, 24h, ....
    time_t limitActionTime; ///< limit action time (leak has been detected)
    int limitPulseAction;   ///< limit action in pulses
    int bypassPin;          ///< number of the bypass pin

    /// Compares two settings
    ///
    /// This method is used for settings comparison (so the
    /// same settings are not applied again, for example)
    ///
    /// \param other the other settings we compare these settings to
    /// \return true if the settings are the same, false otherwise
    bool operator!=(const LeakDetectionConfig_t &other) const {
        return limitResetTime != other.limitResetTime ||
               limitActionTime != other.limitActionTime ||
               limitPulseAction != other.limitPulseAction ||
               bypassPin != other.bypassPin;
    }
};

#endif