#ifndef LIMITS_DEFINITION_H
#define LIMITS_DEFINITION_H

#include "DateTime.h"
#include "Setup.h"

//===============sensor input coversion=================

/// Converts liters to pulses
/// \param l value in liters
/// \return number of pulses
#define LITER_TO_PULSE(l) ((int)(l) / SENSOR_PULSE_IN_LITERS)

/// Converts pulses to liters
/// \param p number of pulses
/// \return number of liters
#define PULSE_TO_LITER(p) (p * SENSOR_PULSE_IN_LITERS)

#define UNUSED 0 ///< unused parameter

//============HIGH-water leak default params============

#define HIGH_LEAK_RESET_TIME	 SEC_TO_MILLIS(10)    ///< high-water leak reset time
#define HIGH_LEAK_LIMIT_ACTION	 LITER_TO_PULSE(300)  ///< high-water leak action (liters)

//============LOW-water leak default params=============

#define LOW_LEAK_RESET_TIME	     SEC_TO_MILLIS(12)    ///< low-water leak reset time
#define LOW_LEAK_LIMIT_ACTION    SEC_TO_MILLIS(40)    ///< low-water like action time

//============TOTAL-water leak default params===========

#define TOTAL_LEAK_RESET_TIME	 MIN_TO_MILLIS(5)     ///< total-water leak reset time  
#define TOTAL_LEAK_LIMIT_ACTION  LITER_TO_PULSE(400)  ///< total-water leak action (liters)

//==========Passing parameters to constructors==========

/// Passes the default high-water leak detection parameters to the constructor
#define HIGH_LEAK_NORMAL_DEFAULT_CONFIG \
{HIGH_LEAK_RESET_TIME, \
UNUSED, \
HIGH_LEAK_LIMIT_ACTION, \
HIGH_BYPASS_PIN} 

/// Passes the default low-water leak detection parameters to the constructor
#define LOW_LEAK_NORMAL_DEFAULT_CONFIG \
{LOW_LEAK_RESET_TIME, \
LOW_LEAK_LIMIT_ACTION, \
UNUSED, \
LOW_BYPASS_PIN}

/// Passes the default total-water leak detection parameters to the constructor
#define TOTAL_LEAK_NORMAL_DEFAULT_CONFIG \
{TOTAL_LEAK_RESET_TIME, \
UNUSED, \
TOTAL_LEAK_LIMIT_ACTION, \
TOTAL_BYPASS_PIN}

#endif