#ifndef LOGGER_H
#define LOGGER_H

#include "Arduino.h"
#include "vector"
#include "DateTime.h"
#include "TimePeriod.h"
#include "ILoggable.h"
#include "IControllable.h"

#define LOG_PERIOD SEC_TO_MILLIS(1) ///< how often the log will be printed out

//==================Logger class========================

/// \author silhavyj A17B0362P
///
/// This class sepresents a controller for the logging
/// part of the system. It is mainly used for debugging
/// and troubleshooting.
class Logger : public IControllable {

//====================private data======================
private:
    static Logger* instance;            ///< the instance of the class
    std::vector<ILoggable*> logs;       ///< collection of classes that are registered in the logging system
    TimePeriod timePeriod = LOG_PERIOD; ///< time period for recognizing when the log should be printed out

//==================private methods=====================
private:
    /// Constructor of the class
    Logger(){};

    /// Copy constructor of the class
    Logger(Logger const&) {};

    /// Assignment operator of the class
    Logger& operator=(Logger const&) { return *this; }

//==================public methods======================
public:
    /// Returns the instance of the class
    ///
    /// If the instance has not been created, it will 
    /// create it and then return it.
    ///
    /// \return the instance of the class
    static Logger* getInstance();

    /// Adds another class to the logging system
    /// \param log another class to be added
    void addEntity(ILoggable* log);

    /// Updates the class
    /// 
    /// If the logging period has passed, it will go over
    /// all the classes in the collection and print the
    /// logs out to the terminal one by one.
    void update() override;
};

#endif