#ifndef PULSE_GENERATOR_H
#define PULSE_GENERATOR_H

#include "Arduino.h"

#include "DateTime.h"
#include "Setup.h"

//===============PulseGenerator class===================

/// \author silhavyj A17B0362P
///
/// Automatic pulse generator used for unit testing.
///
/// This class provides the functionality of generating pulses
/// on the output pin. The number of the pin is passed into the
/// constructor along with the parameters of the generated pulses.
/// This class is used only when it comes to unit testing.
class PulseGenerator {

//====================private data======================
private:
    time_t delayHigh;  ///< delay of the pulse in HIGH state
    time_t delayLow;   ///< delay of the pulse in LOW state
    int outputPin;     ///< number of the pin where the pulses are generated

    int value;         ///< current state of the output pin (HIGH/LOW)
    int pulseCount;    ///< number of pulses that have been generated so far

    time_t timeLastValueChange;  ///< time when the state of the pin changed for the last time

//==================public methods======================
public:
    /// Creates an instance of the class
    ///
    /// \param delayHigh delay of the pulse in HIGH state
    /// \param delayLow delay of the pulse in LOW state
    /// \param outputPin number of the pin where the pulses are generated
    PulseGenerator(time_t delayHigh, time_t delayLow, int outputPin);

    /// Updates the state of the output pin
    ///
    /// It changes the state of the pin depending if it is
    /// time to change it from HIGH to LOW, or otherwise.
    /// Both parameters #delayHigh and #delayLow are defined via the constructor
    void update();

    /// Resets the pulse generator
    ///
    /// When running unit tests, the generator needs to
    /// be reset every time when moving on to the
    /// next test case (scenario).
    void reset();
};

#endif