#include "Consumption.h"

Consumption::Consumption(PulseCounter *pulseCounter, Type type) {
    this->pulseCounter = pulseCounter;
    this->type = type;

    initialDateTime = DateTime::getInstance()->getDateTime();
    count = 0;
}

void Consumption::update() {
    bool timeToReset = 0;
    RTCDateTime currentDateTime = DateTime::getInstance()->getDateTime();

    // check if it's time to reset the counter as the
    // monitoring period might be over
    switch (type) {
        case DAY:
            // test if the current day is different
            if (currentDateTime.day != initialDateTime.day)
                timeToReset = 1;
            break;
            // test if the current week is different
        case WEEK:
            if (currentDateTime.dayOfWeek == 1 && initialDateTime.dayOfWeek == 7)
                timeToReset = 1;
            break;
            // test if the current month is different
        case MONTH:
            if (currentDateTime.month != initialDateTime.month)
                timeToReset = 1;
            break;
    }
    // if it's time to reset the counter, do so
    if (timeToReset) {
        initialDateTime = DateTime::getInstance()->getDateTime();
        count = 0;
    }
    // if there's a pulse, increment the counter
    if (pulseCounter->isActive())
        count++;
}

int Consumption::getConsumptionCount() const {
    return count;
}