#include "FreeMemoryMeasurement.h"

FreeMemoryMeasurement *FreeMemoryMeasurement::instance = NULL;

#ifdef WEB_SERVER
    String HTML_freeRAM(const FreeMemoryMeasurement& freeMemoryMeasurement);
#endif

FreeMemoryMeasurement::FreeMemoryMeasurement() {
    #ifdef WEB_SERVER
        htmlData[16] = &HTML_freeRAM;
    #endif
}

FreeMemoryMeasurement* FreeMemoryMeasurement::getInstance() {
    if (instance == NULL)
        instance = new FreeMemoryMeasurement;
    return instance;
}

int FreeMemoryMeasurement::getFreeRAM() const {
    char top;
    #ifdef __arm__
        return &top - reinterpret_cast<char*>(sbrk(0));
    #elif defined(CORE_TEENSY) || (ARDUINO > 103 && ARDUINO != 151)
        return &top - __brkval;
    #else  // __arm__
        return __brkval ? &top - __brkval : &top - __malloc_heap_start;
    #endif  // __arm__
}

#ifdef LCD_DISPLAY
    const String FreeMemoryMeasurement::getRow(int row) const {
        switch(row) {
            case 0:
                return String("FREE SRAM");
            case 1:
                return String(getFreeRAM()) + "B"; 
                break;
            case 2:
                return "";
                break;
            case 3:
                return "";
                break;
        }
        return "UNTITLED";
    }
#endif

#ifdef DEBUG
    const String FreeMemoryMeasurement::getLogID() const {
        return "[SRAM]";
    }

    const String FreeMemoryMeasurement::getLogDescription() const {
        return "[" +
            String("FREE=") + String(getFreeRAM()) + "B" +
            "]";
    }
#endif

#ifdef WEB_SERVER
    const String FreeMemoryMeasurement::getHTMLData(const int id) const {
        auto fce = htmlData.find(id);
        if (fce == htmlData.end())
            return HTMLDataSource::UNDEFINED_DATA;
        return fce->second(*this);
    }

    String HTML_freeRAM(const FreeMemoryMeasurement& freeMemoryMeasurement) {
        return String(freeMemoryMeasurement.getFreeRAM());
    }
#endif