#include "LCDController.h"

LCDController* LCDController::instance = NULL;

LCDController::LCDController() {
    lcd.begin(); // start the LCD
    index = 0;   // index of the 1st page
}

LCDController* LCDController::getInstance() {
    if (instance == NULL)
        instance = new LCDController;
    return instance;
}

void LCDController::addItem(IDisplayable *item) {
    items.push_back(item);
}

void LCDController::update() {
    String row;

    // test if it's time to switch
    // on to another page
    if (delay.isActive()) {
        // test if there's actually something
        // to display
        if (items.size() == 0)
            return;

        lcd.clear(); // clear up the LCD

        // display all 4 rows of the current
        // page on the LCD display
        for (int i = 0; i < LCD_ROWS; i++) {
            row = items[index]->getRow(i);
            lcd.setCursor(0, i);
            lcd.print(row);
        }
        // increment the page counter
        index++;
        if (index == (int)items.size())
            index = 0;
    }
}

#ifdef DEBUG
    const String LCDController::getLogID() const { return "[LCD]"; }

    const String LCDController::getLogDescription() const {
        String content = "";
        if (items.size() == 0)
            return content;

        for (int i = 0; i < LCD_ROWS; i++) {
            content += items[index]->getRow(i);
            if (i < LCD_ROWS - 1)
                content += "; ";
        }
        return "[" + content + "]";
    }
#endif