#include "PulseCounter.h"

#ifdef WEB_SERVER
    String HTML_lastPulseTime(const PulseCounter& pulseCounter);
    String HTML_currentTime(const PulseCounter& pulseCounter);
    String HTML_currentDate(const PulseCounter& pulseCounter);
#endif

PulseCounter::PulseCounter(int inputPin) {
    this->inputPin = inputPin;

    // initialize the variables
    lastPulseTime      = 0;
    previousInputValue = LOW;
    pulseDown          = LOW;

    #ifdef WEB_SERVER
        // functions providing data to the webserver
        htmlData[12]  = &HTML_lastPulseTime;
        htmlData[19]  = &HTML_currentTime;
        htmlData[115] = &HTML_currentDate;
    #endif
}

void PulseCounter::update() {
    time_t currentTime    = millis();
    int currentInputValue = digitalRead(inputPin);

    // if the signal has gone from HIGH to LOW, a pulse
    // just occurred
    if (currentInputValue == LOW && previousInputValue == HIGH) {
        lastPulseTime = currentTime;
        pulseDown = HIGH;
    } else {
        pulseDown = LOW;
    }
    previousInputValue = currentInputValue;
}

int PulseCounter::isActive() const {
    return pulseDown;
}

time_t PulseCounter::getLastPulseTime() const {
    return lastPulseTime;
}

#ifdef WEB_SERVER
    const String PulseCounter::getHTMLData(const int id) const {
        auto fce = htmlData.find(id);
        if (fce == htmlData.end())
            return HTMLDataSource::UNDEFINED_DATA;
        return fce->second(*this);
    }

    String HTML_lastPulseTime(const PulseCounter& pulseCounter) {
        time_t difference = millis() - pulseCounter.getLastPulseTime();
        return FORMAT_TIME(difference);
    }

    String HTML_currentTime(const PulseCounter& pulseCounter) {
        return DateTime::getInstance()->getTimeStr();
    }

    String HTML_currentDate(const PulseCounter& pulseCounter) {
        return DateTime::getInstance()->getDateStr();
    }
#endif