#include "PulseGenerator.h"

PulseGenerator::PulseGenerator(time_t delayHigh, time_t delayLow, int outputPin) {
    // stores the values passed through the
    // constructor into member variables
    this->delayHigh = delayHigh;
    this->delayLow = delayLow;
    this->outputPin = outputPin;

    // initially reset (default state)
    reset();
}

void PulseGenerator::reset() {
    // resets the member variables
    // to their default values
    value = LOW;
    timeLastValueChange = 0;
    pulseCount = 0;

    // set the output pin into LOW state
    pinMode(outputPin, OUTPUT);
    digitalWrite(outputPin, value);
}

void PulseGenerator::update() {
    // calculate the time difference
    // between the current time and the time when
    // the state of the pin changed for the last time
    time_t currentTime = millis();
    time_t deltaTime = DELTA_TIME(currentTime, timeLastValueChange);

    // if it is time to change the state of the pin,
    // either from HIGH to LOW or from LOW to HIGH, change it.
    if ((value == LOW && deltaTime >= delayLow) || (value == HIGH && deltaTime >= delayHigh)) {
        // update the values of the member function
        timeLastValueChange = currentTime;
        value = !value;

        // update the state of the output pin
        digitalWrite(outputPin, value);

        // if the state of the pin just went from HIGH to LOW,
        // increment the counter of pulses
        // (another pulse was just generated)
        if (value == LOW) {
            pulseCount++;
        }
    }
}