#include <Arduino.h>
#include <unity.h>

#include <Pins.h>
#include <Time.h>
#include <WebServer.h>

#define BAUD_RATE 9600
#define INITIAL_DELAY SEC_TO_MILLIS(2)

WebServer *webServer = WebServer::getInstance();

void testValidHTTPRequest_1(void) {
    String rq = "GET /?notification=0;0;0;0;0;1;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_2(void) {
    String rq = "GET /?notification=0;0;00;0;0;1;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_3(void) {
    String rq = "GET /?notification=0;0;0;0;0;1;0;0;0;; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_4(void) {
    String rq = "GET /?notification=0;0;0;00;0;1;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_5(void) {
    String rq = "GET /?notification=0;0;0;0;0;165;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_6(void) {
    String rq = "GET /?notification=HEY;0;0;0;0;1;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_7(void) {
    String rq = "GET /?notification= HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_8(void) {
    String rq = "GET /?notification=1;1;1;1;1;1;1;1;1;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_9(void) {
    String rq = "GET /?notification=1;1;1;1;1;1;1;1;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_10(void) {
    String rq = "GET /?notification=1;1;1;1;1;1;1;1;1;; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_11(void) {
    String rq = "GET /?notification=0;0;0;0;0;0;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_12(void) {
    String rq = "GET /?notification=0;0;0;123456799685;0;0;0;0;0;jakub.sil@seznam.cz; HTTP/1.1";
    int result = webServer->processHTTPRequestNotification(rq.indexOf(HTTP_RQ_NOTIFICATION), rq, 10);
    TEST_ASSERT_EQUAL(0, result);
}

void setup() {
    CONFIG_PINS
    Serial.begin(BAUD_RATE);
    while (!Serial)
        ;
    delay(INITIAL_DELAY);
    UNITY_BEGIN();
}

void loop() {
    RUN_TEST(testValidHTTPRequest_1);
    RUN_TEST(testValidHTTPRequest_2);
    RUN_TEST(testValidHTTPRequest_3);
    RUN_TEST(testValidHTTPRequest_4);
    RUN_TEST(testValidHTTPRequest_5);
    RUN_TEST(testValidHTTPRequest_6);
    RUN_TEST(testValidHTTPRequest_7);
    RUN_TEST(testValidHTTPRequest_8);
    RUN_TEST(testValidHTTPRequest_9);
    RUN_TEST(testValidHTTPRequest_10);
    RUN_TEST(testValidHTTPRequest_11);
    RUN_TEST(testValidHTTPRequest_12);
    UNITY_END();
}