#include <Arduino.h>
#include <unity.h>

#include <Pins.h>
#include <Time.h>
#include <WebServer.h>

#define BAUD_RATE 9600
#define INITIAL_DELAY SEC_TO_MILLIS(2)

WebServer *webServer = WebServer::getInstance();

void testValidHTTPRequest_1(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;7;8;9;10;11;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_2(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;7;8;9;10;11;12;13;15 HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_3(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;7;8;9;10;11;12;; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_4(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;7;8;9;10;11;; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_5(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;7;8;9;10;;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_6(void) {
    String rq = "GET /?settings=10;1;2;Abcdfs;5;6;7;8;9;10;;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_7(void) {
    String rq = "GET /?settings=10;1;-2;169;5;6;7;8;9;10;;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_8(void) {
    String rq = "GET /?settings=10;1;2;169;5;6;7;8*5;9;10;;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_9(void) {
    String rq = "GET /?settings=0;0;0;0;0;0;0;0;0;0;0;0; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_10(void) {
    String rq = "GET /?settigs=1;1;1;1;1;1;1;1;1;1;1;1; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_11(void) {
    String rq = "GET / HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_12(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;4294967295;8;9;10;11;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_13(void) {
    String rq = "GET /?settings=100;1;2;4;5;6;4294967294;8;9;10;11;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(1, result);
}

void testValidHTTPRequest_14(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;4294967296;8;9;10;11;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_15(void) {
    String rq = "GET /?settings=10;1;2;4;5;6;4294967292;8;9;10;4294967292294967292;12; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_16(void) {
    String rq = "GET /?settings= HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void testValidHTTPRequest_17(void) {
    String rq = "GET /?settings=10;1;2; HTTP/1.1";
    int result = webServer->processHTTPRequestSettings(rq.indexOf("/?settings="), rq, 12);
    TEST_ASSERT_EQUAL(0, result);
}

void setup() {
    CONFIG_PINS
    Serial.begin(BAUD_RATE);
    while (!Serial)
        ;
    delay(INITIAL_DELAY);
    UNITY_BEGIN();
}

void loop() {
    RUN_TEST(testValidHTTPRequest_1);
    RUN_TEST(testValidHTTPRequest_2);
    RUN_TEST(testValidHTTPRequest_3);
    RUN_TEST(testValidHTTPRequest_4);
    RUN_TEST(testValidHTTPRequest_5);
    RUN_TEST(testValidHTTPRequest_6);
    RUN_TEST(testValidHTTPRequest_7);
    RUN_TEST(testValidHTTPRequest_8);
    RUN_TEST(testValidHTTPRequest_9);
    RUN_TEST(testValidHTTPRequest_10);
    RUN_TEST(testValidHTTPRequest_11);
    RUN_TEST(testValidHTTPRequest_12);
    RUN_TEST(testValidHTTPRequest_13);
    RUN_TEST(testValidHTTPRequest_14);
    RUN_TEST(testValidHTTPRequest_15);
    RUN_TEST(testValidHTTPRequest_16);
    RUN_TEST(testValidHTTPRequest_17);
    UNITY_END();
}