#include <Arduino.h>
#include <ArduinoSTL.h>
#include <unity.h>

#include <Pins.h>
#include <DateTime.h>
#include <PulseGenerator.h>
#include <PulseCounter.h>
#include <HighLeakDetection.h>
#include <LowLeakDetection.h>
#include <TotalLeakDetection.h>
#include <LeakDetectionConfig.h>
#include <ALeakDetectable.h>
#include <LeaksController.h>
#include <FreeMemoryMeasurement.h>

#define BAUD_RATE 9600
#define NOT_USED 0
#define PULSE_SOURCE_PIN A15
#define INITIAL_DELAY SEC_TO_MILLIS(2)

PulseCounter pulseCounter(SENSOR_PIN);
HighLeakDetection *highLeakDetection = NULL;
LowLeakDetection *lowLeakDetection = NULL;
TotalLeakDetection *totalLeakDetection = NULL;
PulseGenerator *pulseGenerator = NULL;
LeaksController *leaksController = LeaksController::getInstance();

int totalNumberOfPulses = %number_of_generated_pulses%;
int expectedResultHigh = %high_leak_detected%;
int expectedResultLow = %low_leak_detected%;
int expectedResultTotal = %total_leak_detected%;

void testResultOfHighDetection() {
    int actual = highLeakDetection->isActive();
    TEST_ASSERT_EQUAL(expectedResultHigh, actual);
}

void testResultOfLowDetection() {
    int actual = lowLeakDetection->isActive();
    TEST_ASSERT_EQUAL(expectedResultLow, actual);
}

void testResultOfTotalDetection() {
    int actual = totalLeakDetection->isActive();
    TEST_ASSERT_EQUAL(expectedResultTotal, actual);
}

void testValveState() {
    int actual = leaksController->getValveState();
    int expected = expectedResultHigh || expectedResultLow || expectedResultTotal;
    TEST_ASSERT_EQUAL(expected, actual);
}

void setup() {
    CONFIG_PINS
    leaksController->setPulseCounter(&pulseCounter);
    Serial.begin(BAUD_RATE);
    while (!Serial)
        ;
    delay(INITIAL_DELAY);
    UNITY_BEGIN();
}

void loop() {
    highLeakDetection = new HighLeakDetection(&pulseCounter, ALeakDetectable::High, {%high_leak_reset_time%, NOT_USED, %high_leak_pulses_limit_action%, HIGH_BYPASS_PIN},{});
    lowLeakDetection = new LowLeakDetection(&pulseCounter, ALeakDetectable::Low, {%low_leak_reset_time%, %low_leak_time_action%, NOT_USED, LOW_BYPASS_PIN}, {});
    totalLeakDetection = new TotalLeakDetection(&pulseCounter, ALeakDetectable::Total, {%total_leak_reset_time%, NOT_USED, %total_leak_pulses_limit_action%, TOTAL_BYPASS_PIN}, {});
    pulseGenerator = new PulseGenerator(%delay_high%, %delay_low%, PULSE_SOURCE_PIN);

    leaksController->addLeakDetection(ALeakDetectable::High, highLeakDetection);
    leaksController->addLeakDetection(ALeakDetectable::Low, lowLeakDetection);
    leaksController->addLeakDetection(ALeakDetectable::Total, totalLeakDetection);

    int numberOfGeneratedPulses = 0;
    while (numberOfGeneratedPulses < totalNumberOfPulses) {
        pulseGenerator->update();
        leaksController->update();
       
        if (pulseCounter.isActive())
            numberOfGeneratedPulses++;
    }
    RUN_TEST(testResultOfHighDetection);
    RUN_TEST(testResultOfLowDetection);
    RUN_TEST(testResultOfTotalDetection);
    RUN_TEST(testValveState);


    delete highLeakDetection;
    delete lowLeakDetection;
    delete totalLeakDetection;
    delete pulseGenerator;

    UNITY_END();
}