import json
import os
import shutil
import subprocess

individual_leak_test_template = open('templates/individual_leak_test.cpp').read()
complete_leak_test_template = open('templates/complete_leak_test.cpp').read()

def read_JSON(filename):
    content = open(filename).read()
    json_data = json.loads(content)
    return json_data

def clean_test_dir():
    sub_folders = [f.path for f in os.scandir('../test') if f.is_dir()]
    for dir_name in list(sub_folders):
        sub_folders.extend(fast_scandir(dir_name))
        shutil.rmtree(dir_name)

def fast_scandir(dir_name):
    sub_folders = [f.path for f in os.scandir(dir_name) if f.is_dir()]
    for dir_name in list(sub_folders):
        sub_folders.extend(fast_scandir(dir_name))
    return sub_folders

def execute_tests():
	p = subprocess.Popen(['platformio', 'test'], cwd='../')
	p.wait()

def fill_code(scenario, type, expected_result):
    code = individual_leak_test_template
    code = code.replace("%number_of_generated_pulses%", str(scenario['number_of_generated_pulses']))
    code = code.replace("%expected_result%", str(expected_result))
    code = code.replace("%delay_high%", str(scenario['delay_high']))
    code = code.replace("%delay_low%", str(scenario['delay_low']))
    code = code.replace("%reset_time%", str(scenario['reset_time']))
    code = code.replace("%bypass%", type.upper())
    code = code.replace("%type%", type.capitalize())

    if type == 'high' or type == 'total':
        code = code.replace("%time_action%", 'NOT_USED')
        code = code.replace("%pulses_limit_action%", str(scenario['pulses_limit_action']))
    elif type == 'low':
        code = code.replace("%time_action%", str(scenario['time_action']))
        code = code.replace("%pulses_limit_action%", 'NOT_USED')
    return code

def create_code_of_individual_tests(data, type):
    index = 1
    for scenario in data['positive']:
        code = fill_code(scenario, type, 1)
        name = 'test_'+ type + '_leak_positive_' + str(index)
        os.mkdir('../test' + '\\' + name)
        open('../test\\' + name + '\\' + name + '.cpp', "w+").write(code)
        index += 1

    index = 1
    for scenario in data['negative']:
        code = fill_code(scenario, type, 0)
        name = 'test_'+ type + '_leak_negative_' + str(index)
        os.mkdir('../test' + '\\' + name)
        open('../test\\' + name + '\\' + name + '.cpp', "w+").write(code)
        index += 1

def create_individual_tests():
    create_code_of_individual_tests(read_JSON('data/high_leak_detection.json'), type='high')
    create_code_of_individual_tests(read_JSON('data/low_leak_detection.json'), type='low')
    create_code_of_individual_tests(read_JSON('data/total_leak_detection.json'), type='total')

def create_complete_tests():
    data = read_JSON('data/complete_leak_detection.json')
    index = 1
    for scenario in data['scenarios']:
        code = complete_leak_test_template
        code = code.replace("%number_of_generated_pulses%", str(scenario['number_of_generated_pulses']))
        code = code.replace("%high_leak_detected%", str(scenario['high_leak_detected']))
        code = code.replace("%low_leak_detected%", str(scenario['low_leak_detected']))
        code = code.replace("%total_leak_detected%", str(scenario['total_leak_detected']))

        code = code.replace("%delay_high%", str(scenario['delay_high']))
        code = code.replace("%delay_low%", str(scenario['delay_low']))
        code = code.replace("%number_of_generated_pulses%", str(scenario['number_of_generated_pulses']))

        code = code.replace("%high_leak_reset_time%", str(scenario['high_leak_reset_time']))
        code = code.replace("%high_leak_pulses_limit_action%", str(scenario['high_leak_pulses_limit_action']))

        code = code.replace("%low_leak_reset_time%", str(scenario['low_leak_reset_time']))
        code = code.replace("%low_leak_time_action%", str(scenario['low_leak_time_action']))

        code = code.replace("%total_leak_reset_time%", str(scenario['total_leak_reset_time']))
        code = code.replace("%total_leak_pulses_limit_action%", str(scenario['total_leak_pulses_limit_action']))

        name = 'complete_leak_test_' + str(index)
        os.mkdir('../test' + '\\' + name)
        open('../test\\' + name + '\\' + name + '.cpp', "w+").write(code)
    
        index += 1

def copy_src_files():
	unit_test_dirs = fast_scandir('../test')
	src_files = os.listdir('../src')
	src_files.remove('main.cpp')
	
	for file_name in src_files:
		full_file_name = os.path.join('../src', file_name)
		if os.path.isfile(full_file_name):
			for unit_test_dir in unit_test_dirs:
				shutil.copy(full_file_name, unit_test_dir)

clean_test_dir()
create_individual_tests()
create_complete_tests()
copy_src_files()
execute_tests()
clean_test_dir()